@@ -1,11 +1,13 @@
 
 ## Coerce Objects #########################################################
 
-as.data.frame.RFspatialPointsDataFrame <- function(x, ...) {
+as.data.frame.RFpointsDataFrame <-
+  as.data.frame.RFspatialPointsDataFrame <- function(x, ...) {
   #str(x); kkkk
   cbind(x@data, x@coords)
 }
-as.data.frame.RFspatialGridDataFrame <- function(x, ...) 
+as.data.frame.RFgridDataFrame <-
+  as.data.frame.RFspatialGridDataFrame <- function(x, ...) 
   spatialGridObject2conventional(x, TRUE)
 
 setAs("RFspatialPointsDataFrame", "data.frame",
@@ -77,6 +79,31 @@ as.vector.RFgridDataFrame <-
     as.vector(as.array.RFgridDataFrame(x))
   }
 
+as.matrix.RFpointsDataFrame <-
+  as.matrix.RFspatialPointsDataFrame <- function(x, ...) {
+  z <- as.array.RFspatialPointsDataFrame(x, ...) 
+  if (length(dim(z)) > 2)
+    stop("the data set cannot be turned into a matrix. Try 'as.array'")
+  z
+}
+as.array.RFpointsDataFrame <-
+  as.array.RFspatialPointsDataFrame <- function(x, ...) {
+  z <- as.matrix(x@data)
+  dim <- c(x@.RFparams$vdim, x@.RFparams$n)
+  if (length(x@.RFparams$T) == 3) dim <- c(x@.RFparams$T[3], dim)
+  dim <- c(length(z) / prod(dim), dim)
+  dim <- dim[dim > 1]
+  if (length(dim) == 1) return(as.vector(z))
+  dim(z) <- dim
+  z
+}
+as.vector.RFpointsDataFrame <-
+  as.vector.RFspatialPointsDataFrame <- function(x, ...) {
+  as.vector(as.array.RFspatialPointsDataFrame(x))
+}
+
+
+
 #setAs("RFgridDataFrame", "matrix", gridtomatrix)
 #setAs("RFspatialGridDataFrame", "matrix", gridtomatrix)
 
@@ -347,6 +374,25 @@ setMethod(f="plot", signature(x="RFdataFrame", y="RFdataFrame"),
                           legend=legend, ...)
           )
 
+setMethod(f="plot", signature(x="RFdataFrame", y="matrix"),
+          definition=function(x, y, nmax = 6,
+            plot.variance = (!is.null(x@.RFparams$has.variance) &&
+                             x@.RFparams$has.variance),legend=TRUE, 
+            ...)
+          plotRFdataFrame(x, y=y, nmax=nmax, plot.variance=plot.variance,
+                          legend=legend, ...)
+          )
+
+
+setMethod(f="plot", signature(x="RFdataFrame", y="data.frame"),
+          definition=function(x, y, nmax = 6,
+            plot.variance = (!is.null(x@.RFparams$has.variance) &&
+                             x@.RFparams$has.variance),legend=TRUE, 
+            ...)
+          plotRFdataFrame(x, y=y, nmax=nmax, plot.variance=plot.variance,
+                          legend=legend, ...)
+          )
+
 
 
 setMethod(f="plot", signature(x="RFspatialDataFrame", y="missing"),
@@ -441,6 +487,29 @@ setMethod(f="plot",
                                  MARGIN.movie = MARGIN.movie,
                                  ...))
 
+setMethod(f="plot",
+          signature(x="RFspatialDataFrame", y="data.frame"),
+	  definition=function(
+            x, y, MARGIN = c(1, 2),
+            MARGIN.slices = NULL,
+            n.slices= if (is.null(MARGIN.slices)) 1 else 10,
+            nmax = 6,
+            plot.variance = (!is.null(x@.RFparams$has.variance) &&
+                             x@.RFparams$has.variance),
+            select.variables, # = 1:vdim,
+            zlim,
+            legend=TRUE,
+            MARGIN.movie = NULL,
+            ...)
+          plotRFspatialDataFrame(x=x, y=y, MARGIN=MARGIN,
+                                 MARGIN.slices=MARGIN.slices,
+                                 n.slices=n.slices, nmax=nmax,
+                                 plot.variance = plot.variance,
+                                 select=select.variables,
+                                 zlim=zlim,
+                                 legend=legend,
+                                 MARGIN.movie = MARGIN.movie,
+                                 ...))
 
 setMethod(f="persp",
           signature(x="RFspatialGridDataFrame"),
@@ -495,142 +564,30 @@ setMethod(f="plot", signature(x="RFspatialPointsDataFrame",
             return(invisible(NULL))
           })
  
-# plotRFgridDataFrame <- function(x, y, nmax, plot.variance, ...)
-# siehe nicht.nachladbar.R
-          
-plotRFdataFrame <-  function(x, y, nmax=6, plot.variance, legend, ...) {
-  ## grid   : sorted = TRUE
-  ## points : sorted = FALSE
-#  Print(close.screen(), dev.cur()); print(dev.list())
 
-  stopifnot(!missing(x))
-  x <- trafo_pointsdata(x)
-  nc <- ncol(x$data)
 
-  if (!missing(y)) {
-    y <- trafo_pointsdata(y)
-    y$data <- rep(y$data, length.out=nrow(y$data) * nc)
-    dim(y$data) <- c(length(y$coords), nc)
-  }
-  has.variance <- !is.null(x$RFparams$has.variance) && x$RFparams$has.variance
-  if (!has.variance) plot.variance <- FALSE
-  n <- min(x$RFparams$n, nmax) + plot.variance
-  vdim <- x$RFparams$vdim
 
-  if (nc < n*vdim) {
-    if (n==1) vdim <- nc else if (vdim==1) n <- nc else {
-      stop("ncol(x@data) does not match 'x@.RFparams'; change 'x@.RFparams'")
-    }
-  }
-
-  
-  graphics <- RFoptions()$graphics
-#  Print(graphics, close.screen(), dev.cur()); print(dev.list())
-  ArrangeDevice(graphics, c(1, n)) ## NIE par vor ArrangeDevice !!!!
-
-#  Print(close.screen(), dev.cur()); print(dev.list())
-
-  always.close <- n > 1 || graphics$always_close_screen
-  if (any(par()$mfcol != c(1,1))) par(mfcol=c(1,1))
-  dots <- list(...)
-  dotnames <- names(dots)
-  if ("bg" %in% dotnames) {
-    par(bg=dots$bg)
-    dots$bg <- NULL
-  }
- 
-  if (!("xlab" %in% dotnames)) dots$xlab <- x$lab$x
-  if (!("type" %in% dotnames)) dots$type <- "l"
-
-  make.small.mar <- ("xlab" %in% dotnames &&
-                     is.null(dots$xlab) && is.null(dots$ylab))
-
-  ## variable names
-
-##  Print(x); lll
-  
-  if (!is.null(x$labdata) && all(nchar(x$labdata)>0))
-    names.vdim <- unlist(lapply(strsplit(x$labdata[1:vdim], ".n"),
-                                FUN=function(li) li[[1]]))
-  else {
-    names.vdim <- paste("variable", 1:vdim)
-    x$labdata <- names.vdim
-  }
-
-  if (n>1){
-    ylab.vec <- c(paste("realization ", 1:(n-plot.variance), sep=""),
-                  if (plot.variance) "kriging variance")
-  } else {
-    ylab.vec <- if (vdim==1) x$colnames else ""
-  }
-
-  if ("ylab" %in% dotnames) {
-    if (!is.null(dots$ylab))
-      ylab.vec[1:length(ylab.vec)] <- dots$ylab
-    dots$ylab <- NULL
-  }
-
-  col <- 1:vdim
-  if ("col" %in% dotnames) {
-    if (!is.null(dots$col))
-      col[1:length(col)] <- dots$col
-    dots$col <- NULL
-  }
-
-  split.screen(c(n,1))
-  
-#  if (always.close) {
-#    close.screen(all.screens=TRUE)
-#    par(mfrow=c(1,1))
-#    split.screen(c(n,1))
+#summary.RandomFieldsReturn <- function(object, ...) {
+#  class(object) <- "summary.RandomFieldsReturn"
+#  object
+#}
+#print.RandomFieldsReturn <- function(x, give.attr=FALSE, ...) {
+#  print.summary.RandomFieldsReturn(summary.RandomFieldsReturn(x, ...))#
+#}
+#
+#print.summary.RandomFieldsReturn <- function(x, give.attr=FALSE, ...) {
+#  #str(x)
+#  class(x) <- "double"
+#  str(x, no.list=TRUE, give.attr=FALSE) #
+#  if (!give.attr) {
+#    cs <- attr(x, "coord_system")
+#    if (!(cs[1] %in% c("auto", "cartesian")) ||
+#        (cs[2] != "keep" && cs[1] != cs[2])) {
+#      cat(' - attr(*, "coord_system")=')
+#      str(cs, no.list=TRUE, give.attr=FALSE) #
+#    }
 #  }
-                
-  for (i in 1:n){
-    screen(i)
-    if (make.small.mar)
-      par(oma=c(3,0,1,1)+.1, mar=c(0,3,0,0))
-    else
-      par(oma=c(4,0,1,1)+.1, mar=c(0,4,0,0))
-    ylab <- ylab.vec[i]
-    
-    if (tmp.idx <- (plot.variance && i==n)){
-      i <- x$RFparams$n + plot.variance
-    }
-
-    do.call(graphics::plot,
-            args=c(dots, list(
-              x=x$coords, y=x$data[ , vdim*(i-1)+1],
-              xaxt="n", yaxt="n", ylab=ylab, col=col[1]))
-            )
-
-    if (!missing(y)) {
-      points(x=y$coords, y=y$data[ , vdim*(i-1)+1], pch=22, col="red")
-    }
-    axis(2)
-    if (tmp.idx) i <- n
-    
-    if(i==n){
-      axis(1, outer=always.close)
-      title(xlab=dots$xlab, outer=TRUE) # always.close) 
-    }
-    else axis(1, labels=FALSE)
-    for (j in 1:vdim){
-      if (j==1) next
-      do.call(graphics::points, quote=TRUE,
-              args=c(dots, list(
-                x=x$coords, y=x$data[ , vdim*(i-1)+j], col=col[j]))
-              )
-      if (!missing(y)) {
-        points(x=y$coords, y=y$data[ , vdim*(i-1)+j], pch=22, col="red")
-      }
-      
-      if (i==1) {
-        if ( (!TRUE || vdim > 1) && legend) {
-          legend("topright", col=col, lty=1, legend = c(names.vdim))
-        }
-      }
-    }
-  }
-  if (always.close) close.screen(all.screens=TRUE)
-}
+#  # class(x) <- "RandomFieldsReturn"
+#  invisible(x)
+#}
 
