@@ -26,33 +26,24 @@ extern "C" {
   
   void RNGtest();
   
-  void GetrfParameters(int *covmaxchar, int *methodmaxchar,
-		       int *typemaxchar,
-		       int *covnr, int *methodnr, int *typenr,
-		       int *maxdim, int *maxmodels);
-  void GetrfParametersI(int *covmaxchar, int *methodmaxchar,
-			int *typemaxchar,
-			int *covnr, int *methodnr, int *typenr,
-			int *maxdim, int *maxmodels);
+  void GetCurrentNrOfModels(int *init, int *nr);
   void GetModelNr(char **name, int *n); /* transforms string "covfct" 
 					   into covnr of CovList */
   void GetModelName(int *nr, char **name, char **nick); /* transforms covnr into string */
   SEXP GetParameterNames(SEXP nr);
   SEXP GetSubNames(SEXP nr);
-  void GetAttr(int *type, int *op, int *monotone, int *finiterange, 
+  void GetAttr(int *nr, int *type, int *op, int *monotone, int *finiterange, 
 	       int * simpleArguments,
-	       int *internal, int *stat, int *iso, int *maxdim, int *vdim);
+	       int *internal, int *stat, int *iso, int *maxdim, int *vdim,
+	       int *includevariants, int*n);
   void GetRange(int *nr, int *dim, int *index, double *params, int *lparam,
 		double *range, int *lrange);
   void GetNrParameters(int *covnr, int* kappas);
-  void PrintModelList(int *local, int *operat, int *nick);
-  void GetModelList(int* idx, int*internal);
-  void PrintMethods();
-  void GetMethodName(int *nr, char **name);
+  void PrintModelList(int *local, int *operat, int *nick); // used in Roger's book
+  //  void GetModelList(int* idx, int*internal); // used in generate.R
   SEXP GetAllModelNames();
-  void GetMethodNr(char **name, int *nr);
-  //void GetGridSize(double*,double *,double *,int *,int *,int *,int *); 
-
+  SEXP GetCoordSystem(SEXP keynr, SEXP oldsystem, SEXP newsystem);
+ 
   /* Natural Scaling -- in contrast to the RFparameters(PracticalRange)
      GENERAL_NATURALSCALING may take the following values (furthermore,
      0: no rescaling, +10 numerical determination allowed
@@ -84,6 +75,7 @@ extern "C" {
   
   SEXP EvaluateModel(SEXP X, SEXP Covnr);
   
+  SEXP GetProcessType(SEXP model_reg, SEXP model);
   // storing : current value
   // printlevel : current value
   // naturalscaling : fixed value
@@ -134,22 +126,20 @@ extern "C" {
 		  double *sum,   // \sum (a-b)^2 / 2
 		  double *sq,   // \sum (a-b)^4 / 4
 		  int *n);
-  void fftVario3D(double *coord,
-		  double *sumvals,
-		  double *nbvals,
-		  double *bin,
-		  int *nbin,
-		  int *lenT,
-		  int *stepT,
-		  int *nstepT,
-		  double *phi,
-		  double *theta,
-		  int *repet,
-		  int *vdim,
-		  double *empvario,	
-		  double *n,
-		  int *segmentEmpVario,
-		  int *pseudo);
+  SEXP fftVario3D(SEXP coord,
+		  SEXP sumvals,
+		  SEXP nbvals,
+		  SEXP bin,
+		  SEXP nbin,
+		  SEXP lenT,
+		  SEXP stepT,
+		  SEXP nstepT,
+		  SEXP phi,
+		  SEXP theta,
+		  SEXP repet,
+		  SEXP vdim,
+		  SEXP segmentEmpVario,
+		  SEXP pseudo);
   
   // kriging methods
   SEXP simpleKriging(SEXP Reg, SEXP Tgiven, SEXP X, SEXP Invcov, SEXP Notna,
@@ -183,14 +173,11 @@ extern "C" {
   void poly_basis_extern(int *Dim, int *Deg, int *powmatrix);
   
   // fractal dimension, 
-  void boxcounting(double *z, int *lx, int * repet, double *factor,
-		   int *eps, int *leps, double *sum);
-  void detrendedfluc(double *dat, int *lx, int *repet, int* boxes,
-		     int *ldfa, double *dfavar, double *varmethvar);
-  void periodogram(double *dat, int *len, int *repet, int *fftm, 
-		   int *part, int *shift, double *lambda);
-  void minmax(double *dat, int *dim, int *ldim, int *boxes, int *lb, 
-	      double *count);
+  SEXP boxcounting(double *z, int *lx, int * repet, double *factor, int *eps);
+  SEXP detrendedfluc(SEXP dat, SEXP lx, SEXP repet, SEXP boxes, SEXP ldfa);
+  SEXP periodogram(SEXP dat, SEXP len, SEXP repet, SEXP fftm, 
+		   SEXP part, SEXP shift);
+  SEXP minmax(SEXP dat, SEXP dim, SEXP ldim, SEXP boxes, SEXP lb, SEXP count);
   
 
   //  void Cov(int *reg, double *result);
@@ -198,18 +185,20 @@ extern "C" {
   // void Variogram(int *reg, double *value);
   // void Pseudovariogram(int* reg, double *result);
   SEXP CovMatrixIntern(SEXP reg, SEXP x, SEXP dist, SEXP grid,
-		       SEXP lx, SEXP result);
+		       SEXP lx, SEXP result, SEXP nonzeros);
 
   SEXP VariogramIntern(SEXP reg, SEXP x, SEXP lx, SEXP result);
   SEXP CovMatrixLoc(SEXP reg, SEXP x, SEXP dist, SEXP xdim, SEXP lx,
-		    SEXP result);
+		    SEXP result, SEXP nonzeros);
   SEXP CovLoc(SEXP reg, SEXP x, SEXP y, SEXP xdim, SEXP lx, SEXP result);
 	      
 
   SEXP CovMatrixSelectedLoc(SEXP reg, SEXP x, SEXP dist, SEXP xdim, SEXP lx,
-			    SEXP selected, SEXP nsel, SEXP result);
+			    SEXP selected, SEXP nsel, SEXP result,
+			    SEXP nonzeros);
 
-  SEXP CovMatrixSelected(SEXP reg, SEXP selected, SEXP nsel, SEXP result);
+  SEXP CovMatrixSelected(SEXP reg, SEXP selected, SEXP nsel, SEXP result,
+			 SEXP nonzeros);
 
   SEXP Delete_y(SEXP reg);
   void DeleteKey(int *reg);
@@ -249,7 +238,7 @@ extern "C" {
   
   
   
-  void distInt(int *x, int*N, int *Genes, double *res);
+  SEXP distInt(SEXP XX, SEXP N, SEXP Genes);
   // void MLEMakeExplicite(double *dist, int *Lx, int *idx, int *totald);
 
 
@@ -259,9 +248,9 @@ extern "C" {
   void GetModelRegister(char **name, int* nr);
   
   void MultiDimRange(int *model_nr, double *natscale);
-  void countelements(int *idx, int *N, int *boxes);
-  void countneighbours(int *Xdim, int *parts, int *Squarelength, int*cumgridlen,
-		       int *boxes, int * neighbours, int *OK);
+  SEXP countelements(SEXP idx, SEXP N, SEXP Totparts);
+  SEXP countneighbours(SEXP Xdim, SEXP parts, SEXP Squarelength, 
+		       SEXP cumgridlen, SEXP boxes);
   SEXP getelements(SEXP Idx, SEXP Xdim, SEXP N, SEXP Cumgridlen, SEXP Boxes);
   SEXP getneighbours(SEXP Xdim, SEXP Parts, SEXP Squarelength, SEXP Cumgridlen,
 		     SEXP Neighbours);
@@ -287,7 +276,7 @@ extern "C" {
   void sleepMilli(int *milli);
   void hostname(char **h, int *i);
   void pid(int *i);
-
+  SEXP getChar();
 
 
 }
